set echo off
set feedback on
set verify on
set termout off
set pause off
set echo on
DROP TABLE GT_PRICE
;
clear screen
spool 1
set termout on
CREATE GLOBAL TEMPORARY 
TABLE GT_PRICE 
(
        PRODUCT_ID              NUMBER (6),
        LIST_PRICE              NUMBER (8,2),
        MIN_PRICE               NUMBER (8,2),
        START_DATE              DATE,
        END_DATE                DATE);

desc gt_price
desc price
--gt table created
pause
clear screen
COMMENT ON TABLE GT_PRICE IS 'Prices (both standard and minimum) of products.  Database tracks both effective dates and expiration dates for prices.';
COMMENT ON COLUMN GT_PRICE.PRODUCT_ID IS 'Product number to which price applies.  Product name found in table PRICE.';
COMMENT ON COLUMN GT_PRICE.LIST_PRICE IS 'Undiscounted price (in U.S. dollars).';
COMMENT ON COLUMN GT_PRICE.MIN_PRICE IS 'Lowest price (in U.S. dollars) that can be offered.';
COMMENT ON COLUMN GT_PRICE.START_DATE IS 'Date which standard and minimum prices go into effect.';
COMMENT ON COLUMN GT_PRICE.END_DATE IS 'Date which standard and minimum price expire.  This value can be left NULL.';
--comments add to table
pause
clear screen
select * from dba_tab_comments where table_name = 'GT_PRICE'
/
select * from dba_col_comments where table_name = 'GT_PRICE'
/
--comments in catalog
pause
clear screen
disconnect 
connect scott/tiger@ora816d
select * from dba_tab_comments where table_name = 'GT_PRICE'
/
--if disconnect and reconnect then comments still there
pause
clear screen 
desc gt_price

alter table gt_price modify(product_id not null)
/
desc gt_price
--not null visible in desc
pause
clear screen 
ALTER TABLE GT_PRICE ADD /* CONSTRAINT GT_PRICE$PRODUCT_ID_NN */
   CHECK (PRODUCT_ID IS NOT NULL);
ALTER TABLE GT_PRICE ADD /* CONSTRAINT GT_START_DATE_NN */
   CHECK (START_DATE IS NOT NULL);
ALTER TABLE GT_PRICE ADD /* CONSTRAINT GT_MIN_PRICE_LESS */
   CHECK (LIST_PRICE IS NULL OR MIN_PRICE IS NULL OR MIN_PRICE <= LIST_PRICE);
ALTER TABLE GT_PRICE ADD /* CONSTRAINT GT_START_DATE_LESS */
   CHECK (END_DATE IS NULL OR START_DATE <= END_DATE);
CREATE UNIQUE INDEX I_GT_PRICE ON GT_PRICE (PRODUCT_ID, START_DATE);
create or replace trigger gt_price_no_delete 
before delete on gt_price
begin
raise_application_error(-20000,'Delete prevented by trigger');
end;
/
--can create constraints and a trigger
pause
clear screen 
insert into gt_price 
select * from price
;
--can insert once
pause
insert into gt_price 
select * from price
;
--second time constraint stops me
pause
clear screen
delete from gt_price
;
--trigger correctly prevents delete
--go back to presentation
spool off