set echo on feedback on verify on termout on pause off 
disconnect
connect scott/tiger@ora816d
spool 4b
drop table gt_item;
CREATE INDEX I_ITEM2 ON ITEM (PRODUCT_ID);
clear screen
create global temporary table gt_item
(
        ORDER_ID                NUMBER (4),
        ITEM_ID                 NUMBER (4),
        PRODUCT_ID              NUMBER (6),
        ACTUAL_PRICE            NUMBER (8,2),
        QUANTITY                NUMBER (8),
        TOTAL                   NUMBER (8,2))
on commit preserve rows;
CREATE UNIQUE INDEX I_GT_ITEM ON GT_ITEM (ORDER_ID, ITEM_ID);
CREATE INDEX I_GT_ITEM2 ON GT_ITEM (PRODUCT_ID);
truncate table gt_item;

insert into gt_item select * from item;
rem have created a GT copy of Item table
pause
clear screen
set autotrace on
analyze table item delete statistics;
rem note no statistics on item table
pause
set pause on

select *
from item
where product_id = '200376'
/

rem without stats this query of normal table uses index
pause
set pause off

select *
from gt_item
where product_id = '200376'
/

rem without stats same query on GT table has similar plan also using index
rem note also that GT tables is one of the few new features that does not force use of the CBO
pause
clear screen
alter session set optimizer_mode = 'ALL_ROWS';
rem lets force use of the cost based optimiser
pause

select *
from item
where product_id = '200376'
/

rem without stats this query of normal table uses index
pause

select *
from gt_item
where product_id = '200376'
/

rem without stats same query on GT table still has similar plan also using index
pause

analyze table item estimate statistics;
analyze table gt_item estimate statistics;
rem now add stats to item table
pause

select *
from item
where product_id = '200376'
/
rem now with stats it full scans the table, no index
rem SO ADDING STATS CHANGED THE PLAN
rem for the purpose of this demo it doesn't matter what the plan is, lets just assume that this really is a better plan.
pause

select *
from gt_item
where product_id = '200376'
/

rem but the gt query is different
pause
set autotrace off 
clear screen
rem so now we will 'fake' some statistics

execute sys.dbms_stats.drop_stat_table('SCOTT', stattab => 'GTSTATS');
analyze table item delete statistics;
clear
execute sys.dbms_stats.create_stat_table('SCOTT', stattab => 'GTSTATS');
pause
desc gtstats
rem so create a table to hold the stats
pause
column statid heading 'Statistics ID' format a20
column type heading 'Object|Type'
column c1   heading 'TC:Table Name|I:Index Name' format a20
column c2   heading 'TI:Partition Name' format a20
column c3   heading 'TI:Subpartition Name' format a20
column c4   heading 'T:Unused|C:Column Name' format a20
column c5   heading 'TIC:Owner' format a20
column n1   heading 'TI:Num Rows|C:Num Distinct'
column n2   heading 'TI:Num Blocks|C:Density'
column n3   heading 'T:Avg Row Len|I:Distinct Keys|C:Num Distinct?'
column n4   heading 'T:Unknown|I:Leaf Blks/Key|C:Num Rows?'
column n5   heading 'T:Unused|I:Data Blks/Key|C:Num Nulls'
column n6   heading 'T:Unused|I:Clus Fac|C:Min Value'
column n7   heading 'T:Unused|I:Index Height|C:Max Value'
column n8   heading 'T:Unused|I:Unknown|C:Avg Col Len'
column n9   heading 'TI:Unused|C:Unknown'
column n10  heading 'TI:Unused|C:Unknown'
column n11  heading 'TI:Unused|C:Unknown'
column n12  heading 'TIC:Unused'
column d1   heading 'Last Analyzed'
column r1   heading 'TI:Unused|C:Min Raw'
column r2   heading 'TI:Unused|C:Max Raw'
column ch1  heading 'TI:Unused|C:Unknown'
rem This is what I think the columns mean
pause

execute sys.dbms_stats.gather_table_stats('SCOTT', 'ITEM', cascade => TRUE, stattab => 'GTSTATS');
execute sys.dbms_stats.export_table_stats('SCOTT', 'ITEM', cascade => TRUE, stattab => 'GTSTATS');
rem now gather the stats and also put them into the table
pause

select type, c1, c4, n1 from gtstats order by 1,2,3;
pause
rem I am now going to put the heap table stats on the GT table
update gtstats set c1 = 'GT_ITEM' where c1 = 'ITEM';
update gtstats set c1 = 'I_GT_ITEM' where c1 = 'I_ITEM';
update gtstats set c1 = 'I_GT_ITEM2' where c1 = 'I_ITEM2';
pause
select type, c1, c4, n1 from gtstats order by 1,2,3;
pause
select table_name, num_rows from user_tables where table_name = 'GT_ITEM';
pause
execute sys.dbms_stats.import_table_stats('SCOTT', 'GT_ITEM', cascade => TRUE, stattab => 'GTSTATS');
--I have now imported the heap table stats to the GT table
pause
select table_name, num_rows from user_tables where table_name = 'GT_ITEM';
--the stats have appeared in user tables
pause
set autotrace on
select *
from item
where product_id = '200376'
/
--this is the original execution plan on the table
pause

select *
from gt_item
where product_id = '200376'
/
--this is the new execution plan on the GT table 
--ITS DIFFERENT
pause
disconnect
connect scott/tiger@ora816d
set autotrace off
select table_name, num_rows from user_tables where table_name = 'GT_ITEM';
--disconnect and reconnect and the stats are still on the table
pause
set autotrace on
select *
from gt_item
where product_id = '200376'
/
--its the same plan but there are no rows
spool off