REM This script performs a test to measure the amount of redo generated by inserting varying numbers of rows
REM into different heap and global temporary tables

REM requires following grants - assumed that script run as scott
grant select on sys.v_$sysstat to scott;
grant select on sys.v_$mystat to scott;
grant select on sys.v_$session to scott;

drop table a0;
drop table a1;
drop table a2;
drop table b0;
drop table b1;
drop table b2;

create table a0
(a number,b number,c number,d number,e number)
tablespace users;

create table a1
(a number,b number,c number,d number,e number)
tablespace users;

create unique index a11 on a1(a) 
tablespace indx;

create table a2
(a number,b number,c number,d number,e number)
tablespace users;

create unique index a21 on a2(a,b) 
tablespace indx;

create index a22 on a2(a,c) 
tablespace indx;

create global temporary table b0
(a number,b number,c number,d number,e number)
ON COMMIT DELETE ROWS;

create global temporary table b1
(a number,b number,c number,d number,e number) 
ON COMMIT DELETE ROWS;

create unique index b11 on b1(a);

create global temporary table b2
(a number,b number,c number,d number,e number)
ON COMMIT DELETE ROWS;

create unique index b21 on b2(a,b);
create index b22 on b2(a,c);

drop table gtstat;

create table gtstat
(type	 varchar2(10)
,numindx number
,numrows number
,redolog number
,blocks  number
);

set serveroutput on buffer 100000

declare
	l_table  INTEGER :=1;
	l_type   VARCHAR2(10);
	l_count  INTEGER;
	l_max    NUMBER;
	l_redo_before INTEGER;
	l_redo_after  INTEGER;
	l_factor NUMBER :=2.0;
	l_fcount INTEGER :=0;
	l_blocks_before INTEGER;
	l_blocks_after INTEGER;
begin
	EXECUTE IMMEDIATE 'TRUNCATE TABLE gtstat';
	WHILE l_table <= 6 LOOP
		l_max := 1;
		l_fcount := 0;
		WHILE l_max <= 1000000 LOOP
			IF l_table=1 THEN
				EXECUTE IMMEDIATE 'TRUNCATE TABLE a0';
				l_type := 'HEAP';
			ELSIF l_table=2 THEN
				EXECUTE IMMEDIATE 'TRUNCATE TABLE b0';
				l_type := 'GT';
			ELSIF l_table=3 THEN
				EXECUTE IMMEDIATE 'TRUNCATE TABLE a1';
				l_type := 'HEAP';
			ELSIF l_table=4 THEN
				EXECUTE IMMEDIATE 'TRUNCATE TABLE b1';
				l_type := 'GT';
			ELSIF l_table=5 THEN
				EXECUTE IMMEDIATE 'TRUNCATE TABLE a2';
				l_type := 'HEAP';
			ELSIF l_table=6 THEN
				EXECUTE IMMEDIATE 'TRUNCATE TABLE b2';
				l_type := 'GT';
			END IF;
			l_count := 0;

			select  NVL(sum(blocks),0)
			into 	l_blocks_before
			from  	v$sort_usage
			where  session_num = 
				(select serial# 
				from  v$session 
				where  sid = 
					(select distinct sid 
					from  v$mystat));

			SELECT 	value
			INTO	l_redo_before
			FROM	v$mystat
			WHERE	statistic# = 99;

			WHILE l_count < l_max LOOP
				IF l_table=1 THEN
					INSERT INTO a0 (a,b,c,d,e) VALUES (l_count,l_max-l_count,l_count,l_count,l_count);
				ELSIF l_table=2 THEN
					INSERT INTO b0 (a,b,c,d,e) VALUES (l_count,l_max-l_count,l_count,l_count,l_count);
				ELSIF l_table=3 THEN
					INSERT INTO a1 (a,b,c,d,e) VALUES (l_count,l_max-l_count,l_count,l_count,l_count);
				ELSIF l_table=4 THEN
					INSERT INTO b1 (a,b,c,d,e) VALUES (l_count,l_max-l_count,l_count,l_count,l_count);
				ELSIF l_table=5 THEN
					INSERT INTO a2 (a,b,c,d,e) VALUES (l_count,l_max-l_count,l_count,l_count,l_count);
				ELSIF l_table=6 THEN
					INSERT INTO b2 (a,b,c,d,e) VALUES (l_count,l_max-l_count,l_count,l_count,l_count);
				END IF;
				l_count := l_count + 1;
			END LOOP;

			SELECT 	value
			INTO	l_redo_after
			FROM	v$mystat
			WHERE	statistic# = 99;

			select  NVL(sum(blocks),0)
			into 	l_blocks_after
			from  	v$sort_usage
			where  session_num = 
				(select serial# 
				from  v$session 
				where  sid = 
					(select distinct sid 
					from  v$mystat));

			COMMIT;
--			dbms_output.put_line(l_type||' '||FLOOR((l_table-1)/2)||' fcount='||l_fcount||' factor='||l_factor||' max='||l_max||' table='||l_table||' redo='||TO_CHAR(l_redo_after-l_redo_before));

			INSERT INTO gtstat (type,numindx,numrows,redolog,blocks) 
			VALUES(	l_type
			,	FLOOR((l_table-1)/2)
			,	l_max
			,	l_redo_after-l_redo_before
			,	l_blocks_after-l_blocks_before
			);
			COMMIT;

			IF l_fcount = 0 THEN
				l_factor := 2;
				l_fcount := 2;
			ELSIF l_fcount = 2 THEN
				l_factor := 2.5;
				l_fcount := 5;
			ELSIF l_fcount = 5 THEN
				l_fcount := 0;
				l_factor := 2;
			ELSE
				l_fcount := 0;
				l_factor := 2;
			END IF;
			l_max := l_max * l_factor;

			IF l_table=1 THEN
				EXECUTE IMMEDIATE 'TRUNCATE TABLE a0';
			ELSIF l_table=2 THEN
				EXECUTE IMMEDIATE 'TRUNCATE TABLE b0';
			ELSIF l_table=3 THEN
				EXECUTE IMMEDIATE 'TRUNCATE TABLE a1';
			ELSIF l_table=4 THEN
				EXECUTE IMMEDIATE 'TRUNCATE TABLE b1';
			ELSIF l_table=5 THEN
				EXECUTE IMMEDIATE 'TRUNCATE TABLE a2';
			ELSIF l_table=6 THEN
				EXECUTE IMMEDIATE 'TRUNCATE TABLE b2';
			END IF;

		END LOOP;
		l_table := l_table + 1;
	END LOOP;
	COMMIT;
end;
/

column gt0indx format 999999999
column gt0indx format 999999999
column gt0indx format 999999999
column heap0indx format 999999999
column heap1indx format 999999999
column heap2indx format 999999999
spool gtstat
select	a0.numrows numrows
,	a0.redolog heap0indx, b0.redolog gt0indx
,	a1.redolog heap1indx, b1.redolog gt1indx
,	a2.redolog heap2indx, b2.redolog gt2indx
from	gtstat a0
,	gtstat a1
,	gtstat a2
,	gtstat b0
,	gtstat b1
,	gtstat b2
where	a1.numrows = a0.numrows
and	a2.numrows = a0.numrows
and	b0.numrows = a0.numrows
and	b1.numrows = a0.numrows
and	b2.numrows = a0.numrows
and	a0.type = 'HEAP'
and	a1.type = 'HEAP'
and	a2.type = 'HEAP'
and	b0.type = 'GT'
and	b1.type = 'GT'
and	b2.type = 'GT'
and	a0.numindx = 0
and	a1.numindx = 1
and	a2.numindx = 2
and	b0.numindx = 0
and	b1.numindx = 1
and	b2.numindx = 2
;
spool off

drop table a0;
drop table a1;
drop table a2;
drop table b0;
drop table b1;
drop table b2;
drop table gtstat;
