rem redosave.sql
disconnect
connect scott/tiger@ora816d

--DROP TABLE gtp;
--DROP TABLE gtd;
--DROP TABLE norm;
CREATE GLOBAL TEMPORARY TABLE gtp (a VARCHAR2(4000)) ON COMMIT preserve ROWS;
CREATE GLOBAL TEMPORARY TABLE gtd (a VARCHAR2(4000)) ON COMMIT delete ROWS;
CREATE TABLE norm (a VARCHAR2(4000));

DROP TABLE results;
CREATE TABLE results
(num_rows NUMBER
,num_bytes NUMBER
,type VARCHAR2(4)
,redo_size NUMBER
);

TRUNCATE TABLE results;
INSERT 	INTO results
(	num_rows, num_bytes, type)
SELECT	r.num, b.num, t.type
FROM	(         SELECT 1 num FROM DUAL 
	UNION ALL SELECT 2 num FROM DUAL 
	UNION ALL SELECT 4 num FROM DUAL 
	UNION ALL SELECT 10 FROM DUAL 
	UNION ALL SELECT 20 FROM DUAL 
	UNION ALL SELECT 40 FROM DUAL 
	UNION ALL SELECT 100 FROM DUAL 
	UNION ALL SELECT 200 FROM DUAL 
	UNION ALL SELECT 400 FROM DUAL 
	UNION ALL SELECT 1000 FROM DUAL
--	UNION ALL SELECT 4000 FROM DUAL 
	) r
,	(         SELECT 0 num FROM DUAL 
	UNION ALL SELECT 1 num FROM DUAL 
	UNION ALL SELECT 4 num FROM DUAL 
	UNION ALL SELECT 10 FROM DUAL 
	UNION ALL SELECT 40 FROM DUAL 
	UNION ALL SELECT 100 FROM DUAL 
	UNION ALL SELECT 400 FROM DUAL 
	UNION ALL SELECT 1000 FROM DUAL
	UNION ALL SELECT 4000 FROM DUAL
	) b
,	(         SELECT 'GTP' type FROM DUAL 
	UNION ALL SELECT 'GTD' type FROM DUAL 
	UNION ALL SELECT 'Norm' FROM DUAL 
	) t
;
COMMIT;

set serveroutput on buffer 1000000000 verify on feedback on lines 120 timing off autotrace off pause off echo off
DECLARE
	CURSOR c_test IS 
	SELECT 	r.* 
	,	RPAD(
			DECODE(SIGN(num_bytes),1,TO_CHAR(TO_DATE(num_bytes,'J'),'jsp'),'zero')
		,	num_bytes,'.') string
	FROM 	results r
--	WHERE 	rownum = 1
--	AND 	num_rows = 10
--	AND 	num_bytes > 0
	ORDER BY num_bytes, num_rows
	;
	p_test c_test%ROWTYPE;
	l_redo_before INTEGER;
	l_redo_after INTEGER;
	l_string VARCHAR2(4000);
	l_count INTEGER;
BEGIN
	OPEN c_test;
	INSERT INTO gtp VALUES('Instantiate');
	INSERT INTO gtd VALUES('Instantiate');
	LOOP
		FETCH c_test INTO p_test;
		EXIT WHEN c_test%NOTFOUND;

		commit;
		EXECUTE IMMEDIATE 'TRUNCATE TABLE GTP';
		EXECUTE IMMEDIATE 'TRUNCATE TABLE GTD';
		EXECUTE IMMEDIATE 'TRUNCATE TABLE norm REUSE STORAGE';
		commit;

		SELECT value INTO l_redo_before
		FROM v$sysstat WHERE name = 'redo size';	

--		sys.dbms_output.put_line(p_test.type||':'||p_test.num_bytes||':'||p_test.num_rows);
--		sys.dbms_output.put_line('Redo Before:'||l_redo_before);

		FOR l_count IN 1..p_test.num_rows LOOP
--			sys.dbms_output.put_line('Counter:'||l_count);
			IF p_test.type = 'GTP' THEN
				INSERT INTO gtp VALUES(p_test.string);
			ELSIF p_test.type = 'GTD' THEN
				INSERT INTO gtd VALUES(p_test.string);
			ELSE 
				INSERT INTO norm VALUES(p_test.string);
			END IF;
		END LOOP;

		SELECT value INTO l_redo_after
		FROM v$sysstat WHERE name = 'redo size';

--		sys.dbms_output.put_line('Redo After:'||l_redo_after);

		UPDATE 	results
		SET 	redo_size = l_redo_after - l_redo_before
		WHERE	num_rows = p_test.num_rows
		AND	num_bytes = p_test.num_bytes
		AND	type = p_test.type
		;

	END LOOP;
	CLOSE c_test;
	commit;
END;
/

--select * from results;
column gtdpct format 999.9
column gtppct format 999.9
select	norm.num_rows, norm.num_bytes
, 	gtd.redo_size gtdredo, gtp.redo_size gtdredo, norm.redo_size normredo
, 	gtd.redo_size/gtd.num_rows gtdrpr, gtp.redo_size/gtp.num_rows gtprpr, norm.redo_size/norm.num_rows normrpr
,	gtd.redo_size/norm.redo_size*100 gtdpct
,	gtp.redo_size/norm.redo_size*100 gtppct
from	results gtd
,	results gtp
,	results norm
where	gtd.type = 'GTD'
and	gtp.type = 'GTP'
and	norm.type = 'Norm'
and	gtd.num_rows = norm.num_rows
and	gtp.num_rows = norm.num_rows
and	gtd.num_bytes= norm.num_bytes
and	gtp.num_bytes= norm.num_bytes
--and	gt.redo_size is not null
--and	norm.redo_size is not null
order by 2,1
;
